# Convert youtube chat json to nico xml
# YouTube のチャットファイル(.live_chat.json)をニコニコのxml形式に変換する関数
# チャットファイルはyoutube-dlやyt-dlpなどでダウンロードしたものの形式で動きます。
# YouTube のチャットファイルプレイヤーが見つからなかったので、似た機能であるニコニコ動画プレイヤーでチャットを再生できるように変換する関数です。
# ニコニコ動画コメントプレイヤーには commeon などを利用してください。

# コマンドプロンプトで、下記のように実行します。
# python "C:\Users\sdt\Desktop\test\convert_chat_to_xml\convert_chat_to_xml.py" "C:\Users\sdt\Desktop\test\_video\20200210 - #03【デモンズソウル】KUSA TABERURU！！！【鈴原るる_にじさんじ】 - [HZavcqYkXfo].live_chat.json"

import datetime, json, sys

# 変換したいjsonのファイルパス
main_file_name = sys.argv[1]


# jsonファイルの読み込み
def import_json(main_file_name):
    path = main_file_name
    json_open = open(path, 'r',encoding="utf-8")
    json_load = []
    decoder = json.JSONDecoder()
    with json_open as f:
        line = f.readline()
        while line:
            json_load.append(decoder.raw_decode(line))
            line = f.readline()
    return json_load

# タイムスタンプが0:00のようになっているので、全て秒に変換
def convert_timetext(time_text):
    time_l = time_text.split(":")
    if len(time_l) == 2:
        mini, sec = time_l
        hour = 0
    elif len(time_l) == 3:
        hour, mini, sec = time_l
    second = (int(hour) * 3600) + (int(mini) * 60) + (int(sec))
    # if second < 0:
    #     second = 0
    time_text = str(second) + "00"
    return time_text

def replace_text(text):
    text = text.replace("<", "&lt;")	# 小なりの記号。タグを表記したいときにも必要
    text = text.replace(">", "&gt;")	# 大なりの記号。タグを表記したいときにも必要
    text = text.replace("&", "&amp;")	# アンパサンド。実体参照で使うため、記号として表示するときに必要
    text = text.replace(" ", " ")	# ノーブレークスペース
    text = text.replace(" ", " ")	# フォントサイズの半分のスペース
    text = text.replace(" ", " ")	# フォントサイズのスペース
    text = text.replace("–", "-")	# フォントサイズ半分のダッシュ
    text = text.replace("—", "-")	# フォントサイズのダッシュ
    text = text.replace("'","&#039;")
    return text

# <chat thread="1640883302" no="1" vpos="704" date="1000000012" date_usec="2" anonymity="1" user_id="hR1wp4m-xxIaU2RgsVMr4zPyzeU" mail="184" leaf="0" premium="1" score="0">うぽつですー</chat>
# <chat thread="1640883302" no="34" vpos="3620" date="1640946329" date_usec="0" anonymity="1" user_id="d2i5gFaKEszpTLj7D-_hjQsD7L8" mail="184 " leaf="0" premium="1" score="0">なんかしょっぱいな、このおにぎり</chat>

def convert_youtube_chat_json_to_nico_xml(main_file_name):
    json_load = import_json(main_file_name)
    # jsonの必要な情報を整形
    line_l = ["<packet>\n"]
    for index, item in enumerate(json_load):
        if not "addChatItemAction" in item[0]['replayChatItemAction']['actions'][0].keys():
            continue
        if not "liveChatTextMessageRenderer" in item[0]['replayChatItemAction']['actions'][0]['addChatItemAction']['item'].keys():
            continue
        render = item[0]['replayChatItemAction']['actions'][0]['addChatItemAction']['item']['liveChatTextMessageRenderer']
        user_name = render['authorName']['simpleText']
        time_text = render['timestampText']['simpleText']
        time_text = convert_timetext(time_text)
        text = ""
        # テキストを整形(絵文字は別の模様。ここはまだ調整必要)
        if "text" in render['message']['runs'][0].keys():
            text = render['message']['runs'][0]['text']
        if len(render['message']['runs'][0]) >= 2:
            text += render['message']['runs'][1]['emoji']
        line_l += ["<chat vpos='%s' no='%s' user_id='%s'> %s </chat>\n" % (time_text, index , replace_text(user_name), replace_text(text))]
    #
    # ファイルに書き込み
    line_l += ["</packet>"]
    new_path = main_file_name.replace(".live_chat.json","") + ".xml"
    with open(new_path, mode='w',encoding="utf-8") as f:
        f.writelines(line_l)


# 実行
if __name__ == "__main__":
    convert_youtube_chat_json_to_nico_xml(main_file_name)
